###########################################################
# Fig 1
###########################################################
library(mgcv)
source("tvr_15May20.R")

dir.name <- "FIGURES Supplement_15May20"
dir.create(dir.name)
pdf_path <- paste0("./", dir.name,"/")

d <- read.csv("us-states_9May20.csv", header=T)
d$date <- as.Date(d$date)
states <- sort(unique(d$state))

T0 <- 7.5
nboot <- 300

annealing <- T
se.fixed <- 0
sr.fixed <- NA
se0.fixed <- 0
sr0.fixed <- NA
sm.start <- .5
sr.start <- .01

i.data <- "deaths"
i.state <- "Wisconsin"
	
if(i.data == "deaths"){
	analysis.threshold <- 100
	initiate.threshold <- 2
	length.threshold <- 8
	range0.threshold <- 3
	k <- 10
	moving.ave <- 1
}else{
	analysis.threshold <- 3000
	initiate.threshold <- 5
	length.threshold <- 10
	range0.threshold <- 30
	k <- 10
	moving.ave <- 1
}
	
if(i.data == "deaths"){
	dd <- d[d$state == i.state, c("deaths", "date")]
	names(dd) <- c("y","date")
}else{
	dd <- d[d$state == i.state, c("cases", "date")]
	names(dd) <- c("y","date")
}	

dat.dif <- c(0, dd$y[-1] - dd$y[-length(dd$y)])
# remove negative changes in cummulative cases
dd$y[dat.dif < 0] <- NA
dat.dif <- c(0, dd$y[-1] - dd$y[-length(dd$y)])
			
Tstart <- min(which(dat.dif >= initiate.threshold))
x <- dat.dif[Tstart:length(dat.dif)]
x <- log(x+.5)
while(is.na(x[length(x)])) {
	x <- x[-length(x)]
	dd <- dd[-nrow(dd),]
}
	
day <- 1:length(x)
dat <- data.frame(x = x, day = day)		
dat$date <- dd$date[-(1:(Tstart-1))]

# assuming a delay of 8 days for cases and 18 days for death
if(i.data == "deaths"){
	dat$dateB <- dat$date - 18
}else{
	dat$dateB <- dat$date - 8
}	

mod.gamm <- try(gamm(x ~ s(day, k = k), data = dat, correlation = corARMA(form = ~ day, p=1, q=1), method = "ML", weights = varExp(value = -1, form = ~fitted(.))), silent = TRUE)
if(inherits(mod.gamm, "try-error")){
	show(i.state)
	next
}
x.fitted.gamm = as.numeric(predict(mod.gamm$gam, newdata=dat))
if(!any(x.fitted.gamm >= log(range0.threshold))) next
range0 <- sort(which(x.fitted.gamm >= log(range0.threshold)))[1]

range1 <- length(x)
range <- range0:range1

if(length(range) <= length.threshold) next

x <- x[range]
dat <- dat[range,]

mod <- tvr(x, se.fixed = se.fixed, sr.fixed = sr.fixed, sr0.fixed = sr0.fixed, se0.fixed = se0.fixed,
		sm.start = sm.start, sr.start = sr.start,  
		annealing = annealing)
mod

mod$sr.fixed <- mod$sr
mod$sm.start <- mod$sm
b <- boot_tvr(mod, nboot=nboot, annealing = annealing, moving.ave = moving.ave)

x.fitted = mod$X.fitted
r.fitted <- mod$r.fitted
Pr.fitted <- mod$Pr.fitted

i.0 <- (moving.ave+1)/2

# plot 
pdf(paste0(pdf_path,"Supplement Fig1.pdf"), height=9, width=6)

	par(mfcol=c(3,1), mai = c(.8,.8,.1,.4), cex=1)
		
	lmax <- length(r.fitted)
	at.ticks <- round(1 + (0:5)/5 * (lmax - 1))
	
	#A
	plot(exp(x), xlab="", ylab = "Number", xaxt = "n", log="y")
	lines(exp(x.fitted))
	axis(side = 1, at = at.ticks, labels = format(dat$date[at.ticks], "%d-%b"), las = 2)
	mtext("A", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	#B
	plot(r.fitted, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "", ylim=c(-.2,.4), lwd = 2, xaxt = "n")
	lines(r.fitted + Pr.fitted^.5, lty=2)
	lines(r.fitted - Pr.fitted^.5, lty=2)
	lines(c(0,100), c(0,0), lty=3)
	axis(side = 1, at = at.ticks, labels = format(dat$dateB[at.ticks], "%d-%b"), las = 2)
	mtext("B", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
	
	#C
	plot(r.fitted, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "", ylim=c(-.2,.4), lwd = 2, xaxt = "n")			
	TT <- length(x) - (moving.ave-1)/2
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper95[i.0:TT], rev(b$r.lower95[i.0:TT])), col="lightgray", border=NA)
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper66[i.0:TT], rev(b$r.lower66[i.0:TT])), col="darkgray", border=NA)
	lines(r.fitted)
	lines(c(0,length(x)), c(0,0), lty=2)
	axis(side = 1, at = at.ticks, labels = format(dat$dateB[at.ticks], "%d-%b"), las = 2)
	mtext("C", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
			
dev.off()

###########################################################
# Fig 2
###########################################################

mod.rev <- tvr(rev(x), se.fixed = se.fixed, sr.fixed = sr.fixed, sr0.fixed = sr0.fixed, se0.fixed = se0.fixed,
		sm.start = sm.start, sr.start = sr.start,  
		annealing = annealing)
mod.rev

mod.rev$sr.fixed <- mod.rev$sr
mod.rev$sm.start <- mod.rev$sm
b.rev <- boot_tvr(mod.rev, nboot=nboot, annealing = annealing, moving.ave = moving.ave)

x.fitted.rev = rev(mod.rev$X.fitted)
r.fitted.rev <- -rev(mod.rev$r.fitted)

r.lower95 <- -rev(b.rev$r.upper95)
r.upper95 <- -rev(b.rev$r.lower95)
r.lower66 <- -rev(b.rev$r.upper66)
r.upper66 <- -rev(b.rev$r.lower66)
r <- -b.rev$r[,length(r.fitted.rev):1]


pdf(paste0(pdf_path,"Supplement Fig2.pdf"), height=6, width=10)

	par(mfcol=c(2,2), mai = c(.8,.8,.1,.4))
		
	lmax <- length(r.fitted)
	at.ticks <- round(1 + (0:5)/5 * (lmax - 1))
	
	#A
	plot(exp(x), xlab="", ylab = "Number", xaxt = "n", log="y")
	lines(exp(x.fitted))
	axis(side = 1, at = at.ticks, labels = format(dat$date[at.ticks], "%d-%b"), las = 2)
	mtext("A", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	#B
	plot(r.fitted, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "", ylim=c(-.2,.4), lwd = 2, xaxt = "n")			
	TT <- length(x) - (moving.ave-1)/2
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper95[i.0:TT], rev(b$r.lower95[i.0:TT])), col="lightgray", border=NA)
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper66[i.0:TT], rev(b$r.lower66[i.0:TT])), col="darkgray", border=NA)
	lines(r.fitted)
	lines(c(0,length(x)), c(0,0), lty=2)
	axis(side = 1, at = at.ticks, labels = format(dat$dateB[at.ticks], "%d-%b"), las = 2)
	mtext("B", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	#C
	plot(exp(x), xlab="", ylab = "Number", xaxt = "n", log="y")
	lines(exp(x.fitted.rev))
	axis(side = 1, at = at.ticks, labels = format(dat$date[at.ticks], "%d-%b"), las = 2)
	mtext("C", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
	
	#D
	plot(r.fitted.rev, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "", ylim=c(-.2,.4), lwd = 2, xaxt = "n")			
	TT <- length(x) - (moving.ave-1)/2
	polygon(c(i.0:TT, TT:i.0), c(r.upper95[i.0:TT], rev(r.lower95[i.0:TT])), col="lightgray", border=NA)
	polygon(c(i.0:TT, TT:i.0), c(r.upper66[i.0:TT], rev(r.lower66[i.0:TT])), col="darkgray", border=NA)
	lines(r.fitted.rev)
	lines(c(0,length(x)), c(0,0), lty=2)
	axis(side = 1, at = at.ticks, labels = format(dat$dateB[at.ticks], "%d-%b"), las = 2)
	mtext("D", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

dev.off()


###########################################################
# Fig 3
###########################################################
state.abbrevs <- read.table(file="state_abbrevs.csv", header=T, sep=",")

dd <- read.table(file="USA_TRV_deaths_15May20.csv", header=T, sep=",")
dd.rev <- read.table(file="USA_revTRV_deaths_15May20.csv", header=T, sep=",")

dd$abbr <- as.factor(state.abbrevs$abbr)
dd$abbr2 <- state.abbrevs$abbr2
dd$lift <- state.abbrevs$lift

dd <- dd[dd$state != "New Hampshire", ]
dd.rev <- dd.rev[dd.rev$state != "New Hampshire", ]

dd.rev <- dd.rev[!is.na(dd[,"r0"]),]
dd <- dd[!is.na(dd[,"r0"]),]

dd$state <- droplevels(dd$state)
dd$abbr <- droplevels(dd$abbr)
dd$abbr2 <- droplevels(dd$abbr2)

pdf(paste0(pdf_path,"Supplement Fig3.pdf"), height=5, width=5)

	par(mfcol=c(1,1), mai = c(.8,.8,.1,.4))
	
	plot(dd$r0.est, dd.rev$r0.est, pch=NA, xlab="r(t) from forward time series", ylab="r(t) from reversed time series")
	text(x=dd$r0.est, y=dd.rev$r0.est, labels=dd$abbr)

dev.off()


###########################################################
# Fig 4
###########################################################

mod <- tvr(x, se.fixed = se.fixed, sr.fixed = sr.fixed, sr0.fixed = sr0.fixed, se0.fixed = se0.fixed,
		sm.start = sm.start, sr.start = sr.start,  
		annealing = annealing)
mod
plot(mod)

lag <- TVR_lag_fit(mod, 5, 2)
mod.lag <- lag$mod

mod.lag
plot(mod.lag)

mod.lag$sr.fixed = mod.lag$sr
b.lag <- boot_tvr(mod.lag, nboot=nboot, annealing = annealing, moving.ave = moving.ave)

x.fitted <- mod.lag$X.fitted
r.fitted <- mod.lag$r.fitted

plot(lag$mod)

# plot 
pdf(paste0(pdf_path,"Supplement Fig4.pdf"), height=6, width=5)

	par(mfcol=c(2,1), mai = c(.8,.8,.1,.4), cex=1)
		
	lmax <- length(r.fitted)
	at.ticks <- round(1 + (0:5)/5 * (lmax - 1))
	
	#A
	plot(exp(x), xlab="", ylab = "Number", xaxt = "n", log="y")
	lines(exp(x.fitted))
	axis(side = 1, at = at.ticks, labels = format(dat$date[at.ticks], "%d-%b"), las = 2)
	mtext("A", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	#B
	plot(r.fitted, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "", ylim=c(-.2,.4), lwd = 2, xaxt = "n")			
	TT <- length(x)
	polygon(c(1:TT, TT:1), c(b.lag$r.upper95[1:TT], rev(b.lag$r.lower95[1:TT])), col="lightgray", border=NA)
	polygon(c(1:TT, TT:1), c(b.lag$r.upper66[1:TT], rev(b.lag$r.lower66[1:TT])), col="darkgray", border=NA)
	lines(r.fitted)
	lines(c(0,length(x)), c(0,0), lty=2)
	axis(side = 1, at = at.ticks, labels = format(dat$dateB[at.ticks], "%d-%b"), las = 2)
	mtext("B", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
			
dev.off()

###########################################################
# Fig 5
###########################################################
d <- read.csv("us-states_9May20.csv", header=T)
d$date <- as.Date(d$date)
states <- sort(unique(d$state))

T0 <- 7.5
nboot <- 300

i.data <- "deaths"
i.state <- "Wisconsin"
	
if(i.data == "deaths"){
	analysis.threshold <- 100
	initiate.threshold <- 2
	length.threshold <- 8
	range0.threshold <- 3
	k <- 10
	moving.ave <- 1
}else{
	analysis.threshold <- 3000
	initiate.threshold <- 5
	length.threshold <- 10
	range0.threshold <- 30
	k <- 10
	moving.ave <- 1
}
	
if(i.data == "deaths"){
	dd <- d[d$state == i.state, c("deaths", "date")]
	names(dd) <- c("y","date")
}else{
	dd <- d[d$state == i.state, c("cases", "date")]
	names(dd) <- c("y","date")
}	

dat.dif <- c(0, dd$y[-1] - dd$y[-length(dd$y)])

# remove negative changes in cummulative cases
dd$y[dat.dif < 0] <- NA
dat.dif <- c(0, dd$y[-1] - dd$y[-length(dd$y)])
			
Tstart <- min(which(dat.dif >= initiate.threshold))
x <- dat.dif[Tstart:length(dat.dif)]
x <- log(x+.5)
while(is.na(x[length(x)])) {
	x <- x[-length(x)]
	dd <- dd[-nrow(dd),]
}
	
day <- 1:length(x)
dat <- data.frame(x = x, day = day)		
dat$date <- dd$date[-(1:(Tstart-1))]

# assuming a delay of 8 days for cases and 18 days for death
if(i.data == "deaths"){
	dat$dateB <- dat$date - 18
}else{
	dat$dateB <- dat$date - 8
}	

mod.gamm <- try(gamm(x ~ s(day, k = k), data = dat, correlation = corARMA(form = ~ day, p=1, q=1), method = "ML", weights = varExp(value = -1, form = ~fitted(.))), silent = TRUE)
if(inherits(mod.gamm, "try-error")){
	show(i.state)
	next
}
x.fitted.gamm = as.numeric(predict(mod.gamm$gam, newdata=dat))
if(!any(x.fitted.gamm >= log(range0.threshold))) next
range0 <- sort(which(x.fitted.gamm >= log(range0.threshold)))[1]

range1 <- length(x)

###############
# There were problems fitting with the full range of data
range1 <- range1 - 21
###############

range <- range0:range1

if(length(range) <= length.threshold) next

x <- x[range]
dat <- dat[range,]

k <- 10
mod <- try(gamm(x ~ s(day, k = k), data = dat, correlation = corARMA(form = ~ day, p=1, q=1), method = "ML", weights = varExp(value = -1, form = ~fitted(.))), silent = TRUE)

x.fitted = predict(mod$gam, newdata=dat)
r.fitted <- c(x.fitted[-1] - x.fitted[-length(x.fitted)], NA)

plot(x)
lines(x.fitted)

b <- boot_tvGAMM(mod = mod, data = dat, nboot = nboot)
	
# plot 
pdf(paste0(pdf_path,"Supplement Fig5.pdf"), height=6, width=5)

	par(mfcol=c(2,1), mai = c(.8,.8,.1,.4), cex=1)
		
	lmax <- length(r.fitted)
	at.ticks <- round(1 + (0:5)/5 * (lmax - 1))
	
	#A
	plot(exp(x), xlab="", ylab = "Number", xaxt = "n", log="y")
	lines(exp(x.fitted))
	axis(side = 1, at = at.ticks, labels = format(dat$date[at.ticks], "%d-%b"), las = 2)
	mtext("A", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	#B
	plot(r.fitted, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "", ylim=c(-.2,.4), lwd = 2, xaxt = "n")			
	TT <- length(x) - 1
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper95[i.0:TT], rev(b$r.lower95[i.0:TT])), col="lightgray", border=NA)
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper66[i.0:TT], rev(b$r.lower66[i.0:TT])), col="darkgray", border=NA)
	lines(r.fitted)
	lines(c(0,length(x)), c(0,0), lty=2)
	axis(side = 1, at = at.ticks, labels = format(dat$dateB[at.ticks], "%d-%b"), las = 2)
	mtext("B", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
			
dev.off()

###########################################################
# Fig 6
###########################################################

T0 <- 7.5
nboot <- 300

annealing <- T
se.fixed <- 0
sr.fixed <- NA
se0.fixed <- 0
sr0.fixed <- NA
sm.start <- .5
sr.start <- .01

i.data <- "deaths"
i.state <- "Wisconsin"
	
if(i.data == "deaths"){
	analysis.threshold <- 100
	initiate.threshold <- 2
	length.threshold <- 8
	range0.threshold <- 2
	k <- 10
	moving.ave <- 1
}else{
	analysis.threshold <- 3000
	initiate.threshold <- 5
	length.threshold <- 10
	range0.threshold <- 30
	k <- 10
	moving.ave <- 1
}
	
if(i.data == "deaths"){
	dd <- d[d$state == i.state, c("deaths", "date")]
	names(dd) <- c("y","date")
}else{
	dd <- d[d$state == i.state, c("cases", "date")]
	names(dd) <- c("y","date")
}	

dat.dif <- c(0, dd$y[-1] - dd$y[-length(dd$y)])

# remove negative changes in cummulative cases
dd$y[dat.dif < 0] <- NA
dat.dif <- c(0, dd$y[-1] - dd$y[-length(dd$y)])
			
Tstart <- min(which(dat.dif >= initiate.threshold))
x <- dat.dif[Tstart:length(dat.dif)]
x <- log(x+.5)
while(is.na(x[length(x)])) {
	x <- x[-length(x)]
	dd <- dd[-nrow(dd),]
}
	
day <- 1:length(x)
dat <- data.frame(x = x, day = day)		
dat$date <- dd$date[-(1:(Tstart-1))]

# assuming a delay of 8 days for cases and 18 days for death
if(i.data == "deaths"){
	dat$dateB <- dat$date - 18
}else{
	dat$dateB <- dat$date - 8
}	

mod.gamm <- try(gamm(x ~ s(day, k = k), data = dat, correlation = corARMA(form = ~ day, p=1, q=1), method = "ML", weights = varExp(value = -1, form = ~fitted(.))), silent = TRUE)
if(inherits(mod.gamm, "try-error")){
	show(i.state)
	next
}
x.fitted.gamm = as.numeric(predict(mod.gamm$gam, newdata=dat))
if(!any(x.fitted.gamm >= log(range0.threshold))) next
range0 <- sort(which(x.fitted.gamm >= log(range0.threshold)))[1]

range1 <- length(x)
range <- range0:range1

if(length(range) <= length.threshold) next

x <- x[range]
dat <- dat[range,]

mod <- tvr(x, se.fixed = se.fixed, sr.fixed = sr.fixed, sr0.fixed = sr0.fixed, se0.fixed = se0.fixed,
		sm.start = sm.start, sr.start = sr.start,  
		annealing = annealing)
mod

mod$sr.fixed <- mod$sr
mod$sm.start <- mod$sm
b <- boot_tvr(mod, nboot=nboot, annealing = annealing, moving.ave = moving.ave)

x.fitted = mod$X.fitted
r.fitted <- mod$r.fitted
Pr.fitted <- mod$Pr.fitted

i.0 <- (moving.ave+1)/2

# plot 
pdf(paste0(pdf_path,"Supplement Fig6.pdf"), height=6, width=5)

	par(mfcol=c(2,1), mai = c(.8,.8,.1,.4), cex=1)
		
	lmax <- length(r.fitted)
	at.ticks <- round(1 + (0:5)/5 * (lmax - 1))
	
	#A
	plot(exp(x), xlab="", ylab = "Number", xaxt = "n", log="y")
	lines(exp(x.fitted))
	axis(side = 1, at = at.ticks, labels = format(dat$date[at.ticks], "%d-%b"), las = 2)
	mtext("A", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	#B
	plot(r.fitted, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "", ylim=c(-.2,.4), lwd = 2, xaxt = "n")			
	TT <- length(x) - (moving.ave-1)/2
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper95[i.0:TT], rev(b$r.lower95[i.0:TT])), col="lightgray", border=NA)
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper66[i.0:TT], rev(b$r.lower66[i.0:TT])), col="darkgray", border=NA)
	lines(r.fitted)
	lines(c(0,length(x)), c(0,0), lty=2)
	axis(side = 1, at = at.ticks, labels = format(dat$dateB[at.ticks], "%d-%b"), las = 2)
	mtext("B", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
			
dev.off()

###########################################################
# Fig 7
###########################################################

duration <- 25

# Transmission (Li et al., 2020)
m <- 7.5
v <- 3.4^2

v/m^2
shape <- 2.35
gamma(1 + 2/shape)/gamma(1 + 1/shape)^2 - 1
scale <- m/gamma(1 + 1/shape)

pdf.trans <- dweibull(1:duration, shape=shape, scale = scale)
pdf.trans <- pdf.trans/sum(pdf.trans)

# Mortality (Li et al., 2020)
m <- 18.5
v <- 3.4^2

v/m^2
shape <- 10
gamma(1 + 2/shape)/gamma(1 + 1/shape)^2 - 1
scale <- m/gamma(1 + 1/shape)

pdf.mort <- dweibull(1:duration, shape=shape, scale = scale)
pdf.mort <- pdf.mort/sum(pdf.mort)

# Incubation: Ferretti et al. (2020): lognormal mean 5.5 days, sd 2.2 days
m <- 5.5
v <- 2.2^2

s2 <- log(v/m^2 + 1)
mu <- log(m) - s2/2

pdf.incub <- dlnorm(1:duration, meanlog = mu, sdlog = s2^.5)
pdf.incub <- pdf.incub/sum(pdf.incub)

pdf(paste0(pdf_path,"Supplement Fig7.pdf"), height=9, width=6)
	par(mfcol=c(3,1), mai = c(.8,.8,.1,.4), cex=1)
	
	plot(pdf.trans, xlab="Days", ylab = "Transmission probability")
	lines(pdf.trans)
	mtext("A", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	plot(pdf.mort, xlab="Days", ylab = "Mortality probability")
	lines(pdf.mort)
	mtext("B", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	plot(pdf.incub, xlab="Days", ylab = "Detection probability")
	lines(pdf.incub)
	mtext("C", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
dev.off()

###########################################################
# Fig 8
###########################################################

r.shape <- "step.step"
if(r.shape == "step.step"){
	b0 = 4
	b1 = .8
	b2 = 1.5
	Tmax <- 60
	breaks <- as.factor(c(rep(b0, Tmax/3), rep(b1, Tmax/3), rep(b2, Tmax/3)))
	I.init <- 200
	surv <- .98
	h.report <- 0.5
	se <- .2
	sm.incub <- .2
	sm.mort <- .2
	initx <- 1
}

covid.sim <- covid19_sim(Tmax = Tmax, initx = 1, I.init = I.init, b0 = b0, b1 = b1, b2 = b2, surv = .98, se = .2, sm.incub = .2, sm.mort = .2, r.shape = r.shape, show.fig = T)

if(r.shape == "step.step") b <- c(rep(b0, Tmax/3), rep(b1, Tmax/3), rep(b2, Tmax/3))
	
B <- matrix(0,duration,duration)
diag(B[-1,]) <- 1 - (1 - surv) * pdf.mort[1:(duration-1)]

B[1,] <- b0 * pdf.trans * exp(se^2/2)

x.init <- matrix(c(initx, rep(0,(duration - 1))), ncol=1)

x1 <- matrix(c(1, rep(0,(duration - 1))), ncol=1)
R <- NULL
for(t in 1:duration){
	x1 <- B %*% x1
	R <- c(R, x1[1])
	x1[1] <- 0
}
R0 <- sum(R)

r0 <- log(max(abs(eigen(B)$values)))

# mean field simulation
x <- x.init

Ilist <- array(NA, Tmax)
Ilist[1] <- sum(x)

Dlist <- array(NA, Tmax)
Dlist[1] <- 0

rlist <- array(NA, Tmax)
rlist[1] <- r0

for(t in 2:Tmax){
	
	B[1,] <- b[t] * pdf.trans
	# correcting for lognormal
	B[1,] <- B[1,] * exp(se^2/2)
	D <- (1 - surv) * pdf.mort %*% x
	x <- B %*% x
	
	Ilist[t] <- pdf.incub %*% x
	Dlist[t] <- D
	rlist[t] <- log(max(abs(eigen(B)$values)))
}


X <- matrix(NA, 2, Tmax)
x <- 0
while(sum(x) == 0) x <- rpois(n = duration, lambda = I.init * x.init/sum(x.init))
X[,1] <- c(h.report * sum(x), 0)
x0 <- x

r <- rep(NA, Tmax)
r[1] <- r0

for(t in 2:Tmax){
	
	B1 <- b[t] * pdf.trans * exp(rnorm(1, mean = 0, sd = se))
	D <- rbinom(n = duration, size = x, prob = (1 - surv) * pdf.mort * exp(rnorm(1,sd=sm.mort)))
	x <- x - D
	
	I.new <- rpois(n = 1, lambda = B1 %*% x)
	x[2:duration] <- x[1:(duration-1)]
	x[1] <- I.new
	
	C <- rbinom(n = duration, size = x, prob = h.report * pdf.incub * exp(rnorm(1,sd=sm.incub))) 
	X[,t] <- c(sum(C, na.rm=T), sum(D, na.rm=T))
	
	B[1,] <- B1
	r[t] <- log(max(abs(eigen(B)$values)))
}
	
pdf(paste0(pdf_path,"Supplement Fig8.pdf"), height=10, width=6)
	par(mfcol=c(3,1), mai = c(1,1,.1,.4), cex=1.1)
	
	T0 <- 6.5
	plot(exp(T0*r), ylab="b(t) and R0(t)", xlab="Days")
	lines(b, col="black")
	lines(exp(T0*rlist), col="red")
	mtext("A", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
	
	plot(X[2,], ylab="Daily deaths", xlab="Days")
	mtext("B", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)

	plot(X[1,], ylab="Reported cases", xlab="Days")
	points(X[1,1]/h.report, pch=19)
	mtext("C", side=4, cex=1.4, las=1, padj=-5.5, adj=-.2)
	
dev.off()

###########################################################
# Fig 9 & 10
###########################################################
annealing <- T
se.fixed <- 0
sr.fixed <- NA
se0.fixed <- 0
sr0.fixed <- NA
sm.start <- .5
sr.start <- .01
nboot <- 300

i.data <- "deaths"
r.shape <- "step.step"
#r.shape <- "geometric"

if(i.data == "deaths"){
	analysis.threshold <- 100
	initiate.threshold <- 2
	length.threshold <- 8
	range0.threshold <- 3
	k <- 10
	moving.ave <- 1
	I.init <- 200
}else{
	analysis.threshold <- 3000
	initiate.threshold <- 20
	length.threshold <- 10
	range0.threshold <- 30
	k <- 10
	moving.ave <- 1
	I.init <- 200
}

if(r.shape == "step.step"){
	b0 = 4
	b1 = .8
	b2 = 1.5
	Tmax <- 60
	breaks <- as.factor(c(rep(b0, Tmax/3), rep(b1, Tmax/3), rep(b2, Tmax/3)))
}

if(r.shape == "geometric"){
	b0 = 5
	b1 = .5
	b2 = NULL
	Tmax <- 60
}

covid.sim <- covid19_sim(Tmax = Tmax, initx = 1, I.init = I.init, b0 = b0, b1 = b1, b2 = b2, surv = .98, se = .2, sm.incub = .2, sm.mort = .2, r.shape = r.shape, show.fig = T)

#cases = row 1, deaths = row 2
if(i.data == "deaths"){
	x.sim <- covid.sim$X[2,]
	day.lag <- 18
}else{
	x.sim <- covid.sim$X[1,-1]
	day.lag <- 8
}
	
Tstart <- min(which(x.sim >= initiate.threshold))
x <- x.sim[Tstart:length(x.sim)]
x <- log(x+.5)
	
day <- 1:length(x)
dat <- data.frame(x = x, day = day)		

mod.gamm <- try(gamm(x ~ s(day, k = k), data = dat, correlation = corARMA(form = ~ day, p=1, q=1), method = "ML", weights = varExp(value = -1, form = ~fitted(.))), silent = TRUE)
if(inherits(mod.gamm, "try-error")){
	next
}
x.fitted.gamm = as.numeric(predict(mod.gamm$gam, newdata=dat))
if(!any(x.fitted.gamm >= log(range0.threshold))) next
range0 <- sort(which(x.fitted.gamm >= log(range0.threshold)))[1]

range1 <- length(x)
range <- range0:range1

Tstart <- Tstart + range0

x <- x[range]
dat <- dat[range,]

mod <- tvr(x, se.fixed = se.fixed, sr.fixed = sr.fixed, sr0.fixed = sr0.fixed,
		sm.start = sm.start, sr.start = sr.start,  
		annealing = annealing)
mod
plot(mod)

###### set sr to its fitted value for bootstrapping
mod$sr.fixed <- mod$sr
mod$sm.start <- mod$sm
b <- boot_tvr(mod, nboot=nboot, annealing = annealing, moving.ave = moving.ave, r.order=c(1,0,0))

x.fitted = mod$X.fitted

r.fitted = mod$r.fitted
if(moving.ave > 1){
	rr <- r.fitted
	if(any(is.na(rr))) rr[is.na(rr)] <- approx(rr, xout = which(is.na(rr)))$y
	r.fitted <- filter(rr, rep(1/moving.ave,moving.ave))
}

i.0 <- (moving.ave+1)/2

if(r.shape == "step.step") {
	pdf(paste0(pdf_path,"Supplement Fig9.pdf"), height=6, width=5)
}else{
	pdf(paste0(pdf_path,"Supplement Fig10.pdf"), height=6, width=5)
}
	par(mfcol=c(2,1), mai = c(.1,1,1,.4), cex=1.1)
	
	plot(exp(x), xlab="", ylab = "Number of deaths", xaxt="n", main="")
	lines(exp(x.fitted))
	mtext("A", side=4, cex=1.4, las=1, padj=-4.5, adj=-.2)
	
	par(mai = c(1,1,.1,.4), cex=1.1)
	plot(r.fitted, type = "l", lty=1, col="black", ylab = "r(t)", xlab = "Days", ylim=c(-.2,.4), lwd = 2)
	
	TT <- length(x) - (moving.ave-1)/2
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper95[i.0:TT], rev(b$r.lower95[i.0:TT])), col="lightgray", border=NA)
	polygon(c(i.0:TT, TT:i.0), c(b$r.upper66[i.0:TT], rev(b$r.lower66[i.0:TT])), col="darkgray", border=NA)
	lines(r.fitted)
	lines(c(0,length(x)), c(0,0), lty=2)
	
	lines(covid.sim$r[-(1:(Tstart-day.lag-1))], col="red")
	mtext("B", side=4, cex=1.4, las=1, padj=-4.5, adj=-.2)

dev.off()
		
###########################################################
# Fig 11-13
###########################################################

file.name1 <- "test_TRV_15May.csv"
file.name2 <- "test_revTRV_15May.csv"

# analyzing results
df1 <- read.table(file = file.name1, sep=",", header = T)
df1 <- df1[!is.na(df1$r0),]

df2 <- read.table(file = file.name2, sep=",", header = T)
df2 <- df2[!is.na(df2$r0),]
temp <- df2$r0.est
df2$r0.est <- -df2$r1.est
df2$r1.est <- -temp


# bias r0
i.data <- "deaths"
name.list <- c("Step","Geometric")
pdf(file=paste0(pdf_path,"Supplement Fig11.pdf"), width=6, height=6)
	par(mfcol=c(2,2), mai=c(1,.2,.4,.1))
	
	df <- df1
	for(i.shape in 1:2){
		r.shape <- c("step.step", "geometric")[i.shape]
		d <- df[df$r.shape == r.shape & df$i.data == i.data,]
		hist(d$r0 - d$r.Tstart, main=paste0("Forward-",name.list[i.shape]), xlab="Estimated - True")
	}
	df <- df2
	for(i.shape in 1:2){
		r.shape <- c("step.step", "geometric")[i.shape]
		d <- df[df$r.shape == r.shape & df$i.data == i.data,]
		hist(d$r0 - d$r.Tstart, main=paste0("Reverse-",name.list[i.shape]), xlab="Estimated - True")
	}
dev.off()

# bias r0.est
i.data <- "deaths"
name.list <- c("Step","Geometric")
pdf(file=paste0(pdf_path,"Supplement Fig12.pdf"), width=6, height=6)
	par(mfcol=c(2,2), mai=c(1,.2,.4,.1))
	
	df <- df1
	for(i.shape in 1:2){
		r.shape <- c("step.step", "geometric")[i.shape]
		d <- df[df$r.shape == r.shape & df$i.data == i.data,]
		hist(d$r0.est - d$r.Tstart, main=paste0("Forward-",name.list[i.shape]), xlab="Estimated - True")
	}
	df <- df2
	for(i.shape in 1:2){
		r.shape <- c("step.step", "geometric")[i.shape]
		d <- df[df$r.shape == r.shape & df$i.data == i.data,]
		hist(d$r0.est - d$r.Tstart, main=paste0("Reverse-",name.list[i.shape]), xlab="Estimated - True")
	}
dev.off()

# bias r0.est
i.data <- "cases"
name.list <- c("Step","Geometric")
pdf(file=paste0(pdf_path,"Supplement Fig13.pdf"), width=6, height=6)
	par(mfcol=c(2,2), mai=c(1,.2,.4,.1))
	
	df <- df1
	for(i.shape in 1:2){
		r.shape <- c("step.step", "geometric")[i.shape]
		d <- df[df$r.shape == r.shape & df$i.data == i.data,]
		hist(d$r0.est - d$r.Tstart, main=paste0("Forward-",name.list[i.shape]), xlab="Estimated - True")
	}
	df <- df2
	for(i.shape in 1:2){
		r.shape <- c("step.step", "geometric")[i.shape]
		d <- df[df$r.shape == r.shape & df$i.data == i.data,]
		hist(d$r0.est - d$r.Tstart, main=paste0("Reverse-",name.list[i.shape]), xlab="Estimated - True")
	}
dev.off()


################################################
# confidence intervals for r0.est against true r for forward analysis
df <- df1	
d.table <- data.frame(data=rep(c("deaths","cases"), each=2), shape=rep(c("step", "geometric"), times=2), l95=NA, l66=NA, u66=NA, u95=NA)
count <- 0
for(i.data in c("deaths","cases")) for(r.shape in c("step.step", "geometric")){
	count <- count+1
	d <- df[df$r.shape == r.shape & df$i.data == i.data,]
	
	bias <- mean(d$r0.est - d$r.Tstart)
	r0 <- d$r0 - bias
	r0u95 <- d$r0.u95 - bias
	r0u66 <- d$r0.u66 - bias
	r0l66 <- d$r0.l66 - bias
	r0l95 <- d$r0.l95 - bias

	d.table[count,3:6] <- c(mean(d$r.Tstart < r0l95, na.rm=T),
		mean(d$r.Tstart < r0l66, na.rm=T),
		mean(d$r.Tstart > r0u66, na.rm=T),
		mean(d$r.Tstart > r0u95, na.rm=T))
}
print(d.table, digits=2)
    # data     shape  l95  l66  u66   u95
# 1 deaths      step 0.01 0.12 0.24 0.061
# 2 deaths geometric 0.01 0.22 0.28 0.130
# 3  cases      step 0.04 0.18 0.20 0.050
# 4  cases geometric 0.03 0.24 0.24 0.091

################################################
# confidence intervals for r0 against true r for forward analysis
df <- df1	
d.table <- data.frame(data=rep(c("deaths","cases"), each=2), shape=rep(c("step", "geometric"), times=2), l95=NA, l66=NA, u66=NA, u95=NA)
count <- 0
for(i.data in c("deaths","cases")) for(r.shape in c("step.step", "geometric")){
	count <- count+1
	d <- df[df$r.shape == r.shape & df$i.data == i.data,]
	
	bias <- mean(d$r0 - d$r.Tstart)
	r0 <- d$r0 - bias
	r0u95 <- d$r0.u95 - bias
	r0u66 <- d$r0.u66 - bias
	r0l66 <- d$r0.l66 - bias
	r0l95 <- d$r0.l95 - bias

	d.table[count,3:6] <- c(mean(d$r.Tstart < r0l95, na.rm=T),
		mean(d$r.Tstart < r0l66, na.rm=T),
		mean(d$r.Tstart > r0u66, na.rm=T),
		mean(d$r.Tstart > r0u95, na.rm=T))
}
print(d.table, digits=2)
    # data     shape  l95   l66  u66  u95
# 1 deaths      step 0.01 0.092 0.27 0.10
# 2 deaths geometric 0.01 0.120 0.29 0.18
# 3  cases      step 0.04 0.180 0.21 0.05
# 4  cases geometric 0.03 0.222 0.26 0.11


###########################################################
# Fig 14
###########################################################
state.abbrevs <- read.table(file="state_abbrevs.csv", header=T, sep=",")

dd <- read.table(file="USA_TRV_deaths_15May20.csv", header=T, sep=",")
dd.c <- read.table(file="USA_TRV_cases_15May20.csv", header=T, sep=",")

dd$abbr <- as.factor(state.abbrevs$abbr)
dd$abbr2 <- state.abbrevs$abbr2
dd$lift <- state.abbrevs$lift

dd <- dd[dd$state != "New Hampshire", ]
dd.c <- dd.c[dd.c$state != "New Hampshire", ]

dd.c <- dd.c[!is.na(dd[,"r0"]),]
dd <- dd[!is.na(dd[,"r0"]),]

dd$state <- droplevels(dd$state)
dd$abbr <- droplevels(dd$abbr)
dd$abbr2 <- droplevels(dd$abbr2)

pdf(paste0(pdf_path,"Supplement Fig14.pdf"), height=5, width=5)

	par(mfcol=c(1,1), mai = c(.8,.8,.1,.4))
	
	plot(dd$r0.est, dd.c$r0.est, pch=NA, xlab="r0 from death data", ylab="r0 from case data", )
	lines(c(0,1), c(0,1), lty=2)
	text(x=dd$r0.est, y=dd.c$r0.est, labels=dd$abbr)

dev.off()


