# Anthony R. Ives

####################################################
# Begin simulate_tvGAMM
simulate_tvGAMM <- function(mod, nsim = 1) {
	
	x.fitted.na = predict(mod$gam, newdata=mod$lme$data)

	V <- corMatrix(mod$lme$modelStruct$corStruct)
	var.exp <- coef(mod$lme$modelStruct)
	sd.exp <- var.exp[length(var.exp)]
	sd <- exp(sd.exp * x.fitted.na)
	C <- diag(sigma(mod$lme) * sd)
	invV <- chol(C %*% V %*% C)
	
	e.sim <- matrix(NA, nsim, length(x.fitted.na))
	x.sim <- matrix(NA, nsim, length(x.fitted.na))
	for(i.sim in 1:nsim){
		e.sim[i.sim,] <- t(invV %*% rnorm(length(x.fitted.na)))
		x.sim[i.sim,] <- x.fitted.na + e.sim[i.sim,]
	}

	return(list(x = x.sim,  e.sim = e.sim))
}
# End simulate_tvGAMM
####################################################


####################################################
# Begin boot_trv
boot_tvGAMM <- function(mod, data, nboot = 100) {
	x <- data$x
	x.fitted = predict(mod$gam, newdata=data)
	r.fitted <- c(x.fitted[-1] - x.fitted[-length(x.fitted)], NA)
	Tmax <- length(x.fitted)

	X.sim <- simulate_tvGAMM(mod = mod, nsim = nboot)$x
	
	sim.data <- mod$lme$data
	nonconverg.count <- 0
	X <- matrix(NA, nboot, Tmax)
	r <- matrix(NA, nboot, Tmax)
	i.boot <- 0
	while(i.boot < nboot) {
		i.boot <- i.boot + 1
		if(i.boot %% 100 == 0) show(i.boot)
		sim.data$x <- X.sim[i.boot,]
		    
		mod.b <- try(gamm(formula(mod$gam), data = sim.data, correlation = corARMA(form = ~ day, p=1, q=1), weights = varExp(value = 1, form = ~fitted(.)), method = "REML"), silent = TRUE)
		if(!inherits(mod.b, "try-error")){
			X.fit = predict(mod.b$gam, newdata=data)
			X[i.boot,] <- X.fit
			r[i.boot,-Tmax] <- X.fit[-1] - X.fit[-length(X.fit)]			
		}else{
			nonconverg.count <- nonconverg.count + 1
			X.sim[i.boot,] <- simulate_tvGAMM(mod = mod, nsim = 1)$x
			i.boot <- i.boot - 1
		}
	}
	r0.est <- mean(r[,1], na.rm=T)
	r0.se <- sd(r[,1], na.rm=T)
	r0.bias <- r0.est - r.fitted[1]
	
	r1.est <- mean(r[,Tmax-1], na.rm=T)
	r1.se <- sd(r[,Tmax-1], na.rm=T)
	r1.bias <- r1.est - r.fitted[Tmax-1]
	
	r.upper66 <- array(NA, Tmax)
	r.lower66 <- array(NA, Tmax)
	for(t in 1:(Tmax - 1)){
		r.upper66[t] <- sort(r[,t])[ceiling((1-.3333/2)*nboot)]
		r.lower66[t] <- sort(r[,t])[floor(.3333/2*nboot)]
	}

	if(nboot >= 100){
		r.upper95 <- array(NA, Tmax)
		r.lower95 <- array(NA, Tmax)
		for(t in 1:(Tmax - 1)){
			r.upper95[t] <- sort(r[,t])[ceiling(.975*nboot)]
			r.lower95[t] <- sort(r[,t])[floor(.025*nboot)]
		}
	}else{
		r.upper95 <- NULL
		r.lower95 <- NULL
	}		
	
	return(list(X = X, r = r, X.sim = X.sim, r0.est = r0.est, r0.se = r0.se, r0.bias = r0.bias, r1.est = r1.est, r1.se = r1.se, r1.bias = r1.bias, r.upper66 = r.upper66, r.lower66 = r.lower66, r.upper95 = r.upper95, r.lower95 = r.lower95, nonconverg.count = nonconverg.count))
}
# End boot_tvGAMM
####################################################


####################################################
GAMM_lag_fit <- function(mod, data, min.length, step){
	x <- data$x
	newdata <- data
	mod.aic <- mod
	
	range1 <- seq(from=min.length + 1, to=length(x) - min.length, by=step)
	aic <- c(1,1,10^10)
	
	for(lag1 in range1){
		if(lag1 >= (length(x) - 2*min.length)){
			
			data$day1 <- data$day - lag1
			data$day1[1:lag1] <- 0
			mod0 <- try(gamm(x ~ s(day, k = k) + day1, data = data, correlation = corARMA(form = ~ day, p=1, q=1), method = "ML", weights = varExp(value = 1, form = ~fitted(.))), silent = TRUE)
			if(inherits(mod0, "try-error")){
				show(lag1)
			}else{
				if(aic[3] > AIC(mod0$lme)) {
					aic <- c(lag1, NA, AIC(mod0$lme))
					mod.aic <- mod0
					newdata <- data
				}
			}
		}else{
			range2 <- seq(from=lag1 + min.length , to=length(x) - min.length, by=step)
			for(lag2 in range2){
				data$day1 <- data$day - lag1
				data$day1[1:lag1] <- 0
				
				data$day2 <- data$day - lag2
				data$day2[1:lag2] <- 0
				mod0 <- try(gamm(x ~ s(day, k = k) + day1 + day2, data = data, correlation = corARMA(form = ~ day, p=1, q=1), method = "ML", weights = varExp(value = 1, form = ~fitted(.))), silent = TRUE)
				if(inherits(mod0, "try-error")){
					show(c(lag1,lag2))
				}else{
					if(aic[3] > AIC(mod0$lme)) {
						aic <- c(lag1, lag2, AIC(mod0$lme))
						mod.aic <- mod0
						newdata <- data
					}
				}
			}
		}
	}
	return(list(mod = mod.aic, aic=aic, data=newdata))
}
####################################################


####################################################
# Begin boot_tvGAMM_lag
boot_tvGAMM_lag <- function(mod, data, nboot = 100, min.length, step) {
	x <- data$x
	x.fitted = predict(mod$gam, newdata=data)
	r.fitted <- c(x.fitted[-1] - x.fitted[-length(x.fitted)], NA)
	Tmax <- length(x.fitted)

	X.sim <- simulate_tvGAMM(mod = mod, nsim = nboot)$x
	
	sim.data <- data
	nonconverg.count <- 0
	X <- matrix(NA, nboot, Tmax)
	r <- matrix(NA, nboot, Tmax)
	aic.b <- matrix(NA, nboot, 3)
	i.boot <- 0
	while(i.boot < nboot) {
		i.boot <- i.boot + 1
		if(i.boot %% 10 == 0) show(i.boot)
		sim.data$x[!is.na(sim.data$x)] <- X.sim[i.boot,]
		    
		z.aic.boot <- GAMM_lag_fit(mod=mod, data=sim.data, min.length, step)
		mod.b <- z.aic.boot$mod
		aic.b[i.boot,] <- z.aic.boot$aic
		if(!inherits(mod.b, "try-error")){
			X.fit = predict(mod.b$gam, newdata=z.aic.boot$data)
			X[i.boot,] <- X.fit
			r[i.boot,-Tmax] <- X.fit[-1] - X.fit[-length(X.fit)]			
		}else{
			nonconverg.count <- nonconverg.count + 1
			X.sim[i.boot,] <- simulate_tvGAMM(mod = mod, nsim = 1)$x
			i.boot <- i.boot - 1
		}
	}
	r0.est <- mean(r[,1], na.rm=T)
	r0.se <- sd(r[,1], na.rm=T)
	r0.bias <- r0.est - r.fitted[1]
	
	r1.est <- mean(r[,Tmax-1], na.rm=T)
	r1.se <- sd(r[,Tmax-1], na.rm=T)
	r1.bias <- r1.est - r.fitted[Tmax-1]
	
	r.upper66 <- array(NA, Tmax)
	r.lower66 <- array(NA, Tmax)
	for(t in 1:(Tmax - 1)){
		r.upper66[t] <- sort(r[,t])[ceiling((1-.3333/2)*nboot)]
		r.lower66[t] <- sort(r[,t])[floor(.3333/2*nboot)]
	}

	if(nboot >= 100){
		r.upper95 <- array(NA, Tmax)
		r.lower95 <- array(NA, Tmax)
		for(t in 1:(Tmax - 1)){
			r.upper95[t] <- sort(r[,t])[ceiling(.975*nboot)]
			r.lower95[t] <- sort(r[,t])[floor(.025*nboot)]
		}
	}else{
		r.upper95 <- NULL
		r.lower95 <- NULL
	}		
	
	return(list(X = X, r = r, X.sim = X.sim, r0.est = r0.est, r0.se = r0.se, r0.bias = r0.bias, r1.est = r1.est, r1.se = r1.se, r1.bias = r1.bias, r.upper66 = r.upper66, r.lower66 = r.lower66, r.upper95 = r.upper95, r.lower95 = r.lower95, aic = aic.b, nonconverg.count = nonconverg.count))
}
# End boot_tvGAMM_lag
####################################################

